@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -141,6 +141,8 @@ gst_wavpack_dec_init (GstWavpackDec * dec)
   dec->context = NULL;
   dec->stream_reader = gst_wavpack_stream_reader_new ();
 
+  gst_audio_decoder_set_needs_format (GST_AUDIO_DECODER (dec), TRUE);
+
   gst_wavpack_dec_reset (dec);
 }
 
