@@ -68,6 +68,8 @@
 #include "gstrtpjpegpay.h"
 #include "gstrtpL16depay.h"
 #include "gstrtpL16pay.h"
+#include "gstrtpL24depay.h"
+#include "gstrtpL24pay.h"
 #include "gstasteriskh263.h"
 #include "gstrtpmp1sdepay.h"
 #include "gstrtpmp2tdepay.h"
@@ -95,6 +97,8 @@
 #include "gstrtpvp8pay.h"
 #include "gstrtpvrawdepay.h"
 #include "gstrtpvrawpay.h"
+#include "gstrtpstreampay.h"
+#include "gstrtpstreamdepay.h"
 
 static gboolean
 plugin_init (GstPlugin * plugin)
@@ -236,6 +240,12 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_L16_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_L24_pay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_L24_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_asteriskh263_plugin_init (plugin))
     return FALSE;
 
@@ -317,6 +327,12 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_vraw_pay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_stream_pay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_stream_depay_plugin_init (plugin))
+    return FALSE;
+
   return TRUE;
 }
 
