@@ -257,12 +257,13 @@ gst_rtp_gst_pay_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+#define RTP_HEADER_LEN 12
 
 static gboolean
 gst_rtp_gst_pay_create_from_adapter (GstRtpGSTPay * rtpgstpay,
     GstClockTime timestamp)
 {
-  guint avail;
+  guint avail, mtu;
   guint frag_offset;
   GstBufferList *list;
 
@@ -270,7 +271,9 @@ gst_rtp_gst_pay_create_from_adapter (GstRtpGSTPay * rtpgstpay,
   if (avail == 0)
     return FALSE;
 
-  list = gst_buffer_list_new ();
+  mtu = GST_RTP_BASE_PAYLOAD_MTU (rtpgstpay);
+
+  list = gst_buffer_list_new_sized ((avail / (mtu - (RTP_HEADER_LEN + 8))) + 1);
   frag_offset = 0;
 
   while (avail) {
@@ -287,7 +290,7 @@ gst_rtp_gst_pay_create_from_adapter (GstRtpGSTPay * rtpgstpay,
     packet_len = gst_rtp_buffer_calc_packet_len (8 + avail, 0, 0);
 
     /* fill one MTU or all available bytes */
-    towrite = MIN (packet_len, GST_RTP_BASE_PAYLOAD_MTU (rtpgstpay));
+    towrite = MIN (packet_len, mtu);
 
     /* this is the payload length */
     payload_len = gst_rtp_buffer_calc_payload_len (towrite, 0, 0);
@@ -353,15 +356,16 @@ static GstFlowReturn
 gst_rtp_gst_pay_flush (GstRtpGSTPay * rtpgstpay, GstClockTime timestamp)
 {
   GstFlowReturn ret = GST_FLOW_OK;
-  GList *iter, *next;
+  GList *iter;
 
   gst_rtp_gst_pay_create_from_adapter (rtpgstpay, timestamp);
-  for (iter = rtpgstpay->pending_buffers; iter; iter = next) {
+
+  iter = rtpgstpay->pending_buffers;
+  while (iter) {
     GstBufferList *list = iter->data;
 
-    next = iter->next;
-    rtpgstpay->pending_buffers = g_list_remove_link (rtpgstpay->pending_buffers,
-        iter);
+    rtpgstpay->pending_buffers = iter =
+        g_list_delete_link (rtpgstpay->pending_buffers, iter);
 
     /* push the whole buffer list at once */
     ret = gst_rtp_base_payload_push_list (GST_RTP_BASE_PAYLOAD (rtpgstpay),
@@ -410,6 +414,9 @@ gst_rtp_gst_pay_send_caps (GstRtpGSTPay * rtpgstpay, guint8 cv, GstCaps * caps)
   guint capslen;
   GstBuffer *outbuf;
 
+  if (rtpgstpay->flags & (1 << 7))
+    return;
+
   capsstr = gst_caps_to_string (caps);
   capslen = strlen (capsstr);
   /* for 0 byte */
