@@ -256,7 +256,7 @@ rtp_source_init (RTPSource * src)
   src->payload = -1;
   src->clock_rate = -1;
   src->packets = g_queue_new ();
-  src->seqnum_base = -1;
+  src->seqnum_offset = -1;
   src->last_rtptime = -1;
 
   src->retained_feedback = g_queue_new ();
@@ -341,7 +341,7 @@ rtp_source_create_stats (RTPSource * src)
       "received-bye", G_TYPE_BOOLEAN, src->marked_bye,
       "is-csrc", G_TYPE_BOOLEAN, src->is_csrc,
       "is-sender", G_TYPE_BOOLEAN, is_sender,
-      "seqnum-base", G_TYPE_INT, src->seqnum_base,
+      "seqnum-base", G_TYPE_INT, src->seqnum_offset,
       "clock-rate", G_TYPE_INT, src->clock_rate, NULL);
 
   /* add address and port */
@@ -761,12 +761,12 @@ rtp_source_update_caps (RTPSource * src, GstCaps * caps)
 
   GST_DEBUG ("got clock-rate %d", src->clock_rate);
 
-  if (gst_structure_get_uint (s, "seqnum-base", &val))
-    src->seqnum_base = val;
+  if (gst_structure_get_uint (s, "seqnum-offset", &val))
+    src->seqnum_offset = val;
   else
-    src->seqnum_base = -1;
+    src->seqnum_offset = -1;
 
-  GST_DEBUG ("got seqnum-base %" G_GINT32_FORMAT, src->seqnum_base);
+  GST_DEBUG ("got seqnum-offset %" G_GINT32_FORMAT, src->seqnum_offset);
 
   gst_caps_replace (&src->caps, caps);
 }
